html {
  margin: 0;
  border: 0;
  padding: 0;
  width: 100%;
  height: 100%;
}

body {
  box-sizing: border-box;
  margin: 0;
  border: 0;
  padding: 0;
  width: 100%;
  height: 100%;
  overflow: hidden;
}

#pinned {
  position: fixed;
  z-index: 100;
  box-sizing: border-box;
  width: 100%;
}

#pinned div {
  box-sizing: border-box;
  margin: 0.25em;
  border-radius: 0.5em;
  padding: 0.75em;
  cursor: pointer;
}

#pinned .error {
  background-color: #FFBABA;
  color: #D8000C;
}

#pinned .warning {
  background-color: #FEEFB3;
  color: #9F6000;
}

#pinned .success {
  background-color: #DFF2BF;
  color: #4F8A10;
}

#pinned .info {
  background-color: #BDE5F8;
  color: #00529B;
}

#editor {
  box-sizing: border-box;
  margin: 0;
  border: none;
  padding: 0;
  width: 100%;
  height: 90%;
  overflow: auto;
}

#toolbar {
  box-sizing: border-box;
  margin: 0;
  border-top: 2px ridge black;
  padding: 0.25em;
  background-color: #eeffee;
  height: 10%;
}
